clear all; close all; clc;

%Soil and karst drip water isotope model used to derive results in:
%       
%       Markowska et al (2020), Geochimica et Cosmochimica Acta,
%       "Modern speleothem oxygen isotope hydroclimate records in
%       water-limited SE Australia" https://doi.org/10.1016/j.gca.2019.12.007
%
%version 08/11/2019
%
%if you use this code or make a derivative, please acknowledge by citing
%the above paper and the following from which the model is derived:
%
%       Cuthbert, Mark O., et al. "Drip water isotopes in semi-arid karst: 
%       implications for speleothem paleoclimatology."
%       Earth and Planetary Science Letters 395 (2014): 194-204.
%       

%% GET or SET INPUT PARAMETERS

%Get daily time-series input of rain and PEt from Excel
input = 'Model_Climate_Input' %spreadsheet with input data
startrow = 2
startdate = '01/01/1900'
numdays = 41728  %number of days in run
day = xlsread(input, strcat('a',num2str(startrow),':a',num2str(numdays+startrow-1)));
rain = xlsread(input, strcat('b',num2str(startrow),':b',num2str(numdays+startrow-1)));
pe = xlsread(input, strcat('c',num2str(startrow),':c',num2str(numdays+startrow-1)));

%set up some pre-dimensioned arrays for efficiency
%rain = zeros(numdays,1);
aet = zeros(numdays,1);
rech = zeros(numdays,1);
msoil = zeros(numdays,1);       
isoil = zeros(numdays,1);              
mkarst = zeros(numdays,1);  
dkarst = zeros(numdays,1);            
ikarst = zeros(numdays,1);           
ikarstStartEvap = zeros(numdays,1);
karstF = zeros(numdays,1);

%Set location for output data
output = 'Isotope_SMBM_out' %spreadsheet for output data

%Parameters for SMBM - as per Cuthbert et al. (2014) EPSL paper
smd = 86.84;        %set initial soil SMD (mm)
ro = 0;             %runoff - assumed zero
thetaFC = 25;       %moisture content at Field Capacity (%)
thetaWP = 5;        %moisture content at Wilting Point (%)
Ze = 0.1;           %Evaporation 'extinction' depth (m)
Kc = 1;             %Crop co-efficient
B = 0.1;            %Proportion of bare soil
P = 0.5;            %FAO P parameter
Zr = 0.47;          %Rooting depth (m)

%Isotope model params - as per most frequent parameters in EPSL paper
%then tweaked for Wello C Stalagmite
isoil0 = -4.0;        %initital store isotopic composition (per mil)
msoil0 = 188;       %initial soil store depth (mm)
soilfrac = 1;       %soil isotopic fractionation factor
mkarst0 = 200;      %initial karst store depth (mm) 200
karstTau = 0.004;    %linear drainage from karst store (/d) 0.03
n = 0.3;            %epikarst fractionation A param
m = -2.0;          %epikarst fractionation A/B param
karstEthresh = 60;  %karst store depth threshold below which fractionation occurs (mm)
ikarst0 = -4.0;       %initial isotopic composition of karst store
karstEvap = 0.04;   %epikarst evaporation rate (mm/d)

%% DO CALCS

%%%%%%%%%%%%%%%%%%%%SMBM calcs%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

TAW = 1000 * ((thetaFC - thetaWP) * Zr * (1 - B) / 100 + (thetaFC - 0.5 * thetaWP) * Ze * B / 100);
RAW = P * TAW;

for d = 1:numdays;
   
smdm1 = smd; %previous days smd
inf = rain(d) - ro;   
pet = pe(d) * Kc;
Ks = (TAW - smd) / (TAW - RAW);

%case for when pet can be met by excess rainfall and/or a decrease in the SMD
if inf > pet;
    aet(d) = pet;
    if inf - pet > smd;
        rech(d) = inf - pet - smd;
        smd = 0;
    else
        smd = smd + pet - inf;
        rech(d) = 0;
    end   %pet>smd
    
else    

rech(d) = 0;

%rest of calcs for when inf < pet

%if crop not stressed
if smd < RAW;
    aet(d) = pet;
    smd = smd + aet(d) - inf;
else
    %if crop stressed
    if smd > RAW;
        if smd < TAW;
        aet(d) = inf + Ks * (pet - inf);
        smd = smd + aet(d) - inf;
        end
    else
        %wilting?
        if smd >= TAW;
            aet(d) = inf;
            smd = TAW;
        end
    end
end
end  

MB = rain(d) - aet(d) - rech(d) - smdm1 + smd;

%%%%%%%%%%%%%%%%Isotope calcs%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

irain(d) = isoil0; %assume constant isotopic compostion of rain

if d == 1;
    msoil(d) = msoil0-smd;  %depth of water in soil (mm)
    isoil(d) = isoil0;  %isotopic composition of store (per mil)
    mkarst(d) = mkarst0;  %depth of water in karst (mm)
    dkarst(d) = mkarst(d)*karstTau + karstEvap; %drainage from karst (mm)
    karstF(d) = 1; %initialise the f parameter
    ikarst(d) = ikarst0; %initial karst store isotopic composition (per mil)
    ikarstStartEvap(d) = ikarst(d);
else %i.e. d>1
    msoil(d) = msoil(d-1) - aet(d-1) + rain(d-1) - rech(d-1); %accounting for moisture in the soil store
    isoil(d) = (isoil(d-1)*msoil(d-1) + rain(d-1)*irain(d-1) - aet(d-1)*isoil(d-1)*soilfrac - rech(d-1)*isoil(d-1))/msoil(d); %accounting for isotope composition
    mkarst(d) = mkarst(d-1) - dkarst(d-1) + rech(d-1); %accounting for moisture in the karst store
    
    %calc drainage from karst store (mm), making sure the store depth doesn't go negative
    if (mkarst(d)*karstTau + karstEvap) > mkarst(d);
        dkarst(d) = mkarst(d);
    else
        dkarst(d) = mkarst(d)*karstTau + karstEvap;
    end
    
    %calc proportion of karst store remaining since last recharge increasing the store above the threshold
    if mkarst(d) < karstEthresh;
       karstF(d) = mkarst(d)/karstEthresh;
    else
       karstF(d) = 1;
    end    
       
    %calc isotopic composition of ikarst at start of fractionation event
    if karstF(d) == 1;
       ikarstStartEvap(d) = ikarst(d-1);
    else
       ikarstStartEvap(d) = ikarstStartEvap(d-1);
    end   
    
       %calc epikarst fractionation if store empty enough, or else just do a normal isotope 'mass balance'
       %output 'ikarst' is dripwater d18O. Kim and O'Neill or other calcite fractionation equation can be added
    if karstF(d) < 1;
       ikarst(d) = (ikarstStartEvap(d)-m)*(karstF(d)^n) + m;
    else
       ikarst(d) = (mkarst(d-1)*ikarst(d-1)+rech(d-1)*isoil(d-1)-dkarst(d-1)*ikarst(d-1))/mkarst(d);
    end
                  
end %end if d = 1

end %time loop

%% OUTPUT DATA for groudnwater recharge and drip water isotopes
xlswrite(output, cellstr('Day'),'results','A1');
xlswrite(output, day,'results','A2');
xlswrite(output, cellstr('Dripwater d18O'),'results','B1');
xlswrite(output, ikarst,'results','B2');
xlswrite(output, cellstr('Recharge (mm/d)'),'results','C1');
xlswrite(output, rech,'results','C2');